@@ -90,9 +90,9 @@ int GZFAM::Zerror(PGLOBAL g)
 
   if (errnum == Z_ERRNO)
 #if defined(_WIN32)
-    snprintf(g->Message, sizeof(g->Message), MSG(READ_ERROR), To_File, strerror(NULL));
+    sprintf(g->Message, MSG(READ_ERROR), To_File, strerror(NULL));
 #else   // !_WIN32
-    snprintf(g->Message, sizeof(g->Message), MSG(READ_ERROR), To_File, strerror(errno));
+    sprintf(g->Message, MSG(READ_ERROR), To_File, strerror(errno));
 #endif  // !_WIN32
 
     return (errnum == Z_STREAM_END) ? RC_EF : RC_FX;
@@ -152,7 +152,7 @@ bool GZFAM::OpenTableFile(PGLOBAL g)
 //      Last = Nrec;              // For ZBKFAM
         Tdbp->ResetSize();
       } else {
-        snprintf(g->Message, sizeof(g->Message), MSG(NO_PART_DEL), "GZ");
+        sprintf(g->Message, MSG(NO_PART_DEL), "GZ");
         return true;
       } // endif filter
 
@@ -161,7 +161,7 @@ bool GZFAM::OpenTableFile(PGLOBAL g)
       strcpy(opmode, "a+");
       break;
     default:
-      snprintf(g->Message, sizeof(g->Message), MSG(BAD_OPEN_MODE), mode);
+      sprintf(g->Message, MSG(BAD_OPEN_MODE), mode);
       return true;
     } // endswitch Mode
 
@@ -174,7 +174,7 @@ bool GZFAM::OpenTableFile(PGLOBAL g)
   Zfile = gzopen(PlugSetPath(filename, To_File, Tdbp->GetPath()), opmode);
 
   if (Zfile == NULL) {
-    snprintf(g->Message, sizeof(g->Message), MSG(GZOPEN_ERROR),
+    sprintf(g->Message, MSG(GZOPEN_ERROR),
             opmode, (int)errno, filename);
     strcat(strcat(g->Message, ": "), strerror(errno));
     return (mode == MODE_READ && errno == ENOENT)
@@ -249,13 +249,13 @@ int GZFAM::GetNextPos(void)
 /***********************************************************************/
 bool GZFAM::SetPos(PGLOBAL g, int pos __attribute__((unused)))
   {
-  snprintf(g->Message, sizeof(g->Message), MSG(NO_SETPOS_YET), "GZ");
+  sprintf(g->Message, MSG(NO_SETPOS_YET), "GZ");
   return true;
 #if 0
   Fpos = pos;
 
   if (fseek(Stream, Fpos, SEEK_SET)) {
-    snprintf(g->Message, sizeof(g->Message), MSG(FSETPOS_ERROR), Fpos);
+    sprintf(g->Message, MSG(FSETPOS_ERROR), Fpos);
     return true;
     } // endif
 
@@ -647,7 +647,7 @@ int ZBKFAM::WriteBuffer(PGLOBAL g)
 int ZBKFAM::DeleteRecords(PGLOBAL g, int irc)
   {
   if (irc == RC_EF) {
-    LPCSTR  name __attribute__((unused)) = Tdbp->GetName();
+    (void) Tdbp->GetName();                     // XXX Should be removed ?
     PDOSDEF defp = (PDOSDEF)Tdbp->GetDef();
 
     defp->SetBlock(0);
@@ -655,7 +655,7 @@ int ZBKFAM::DeleteRecords(PGLOBAL g, int irc)
 
     if (!defp->SetIntCatInfo("Blocks", 0) ||
         !defp->SetIntCatInfo("Last", 0)) {
-      snprintf(g->Message, sizeof(g->Message), MSG(UPDATE_ERROR), "Header");
+      sprintf(g->Message, MSG(UPDATE_ERROR), "Header");
       return RC_FX;
     } else
       return RC_OK;
@@ -673,7 +673,7 @@ void ZBKFAM::CloseTableFile(PGLOBAL g, bool)
   int rc = RC_OK;
 
   if (Tdbp->GetMode() == MODE_INSERT) {
-    LPCSTR  name __attribute__((unused))= Tdbp->GetName();
+    (void) Tdbp->GetName();                     // XXX Should be removed?
     PDOSDEF defp = (PDOSDEF)Tdbp->GetDef();
 
     if (CurNum && !Closing) {
@@ -932,7 +932,7 @@ bool ZLBFAM::AllocateBuffer(PGLOBAL g)
 
 #if defined(NOLIB)
   if (!zlib && LoadZlib()) {
-    snprintf(g->Message, sizeof(g->Message), MSG(DLL_LOAD_ERROR), GetLastError(), "zlib.dll");
+    sprintf(g->Message, MSG(DLL_LOAD_ERROR), GetLastError(), "zlib.dll");
     return TRUE;
     } // endif zlib
 #endif
@@ -964,9 +964,9 @@ bool ZLBFAM::AllocateBuffer(PGLOBAL g)
 
   if (zrc != Z_OK) {
     if (Zstream->msg)
-      snprintf(g->Message, sizeof(g->Message), "%s error: %s", msg, Zstream->msg);
+      sprintf(g->Message, "%s error: %s", msg, Zstream->msg);
     else
-      snprintf(g->Message, sizeof(g->Message), "%s error: %d", msg, zrc);
+      sprintf(g->Message, "%s error: %d", msg, zrc);
 
     return TRUE;
     } // endif zrc
@@ -1017,9 +1017,9 @@ bool ZLBFAM::AllocateBuffer(PGLOBAL g)
         return FALSE;
       case RC_FX:
 #if defined(UNIX)
-        snprintf(g->Message, sizeof(g->Message), MSG(READ_ERROR), To_File, strerror(errno));
+        sprintf(g->Message, MSG(READ_ERROR), To_File, strerror(errno));
 #else
-        snprintf(g->Message, sizeof(g->Message), MSG(READ_ERROR), To_File, _strerror(NULL));
+        sprintf(g->Message, MSG(READ_ERROR), To_File, _strerror(NULL));
 #endif
         /* falls through */
       case RC_NF:
@@ -1028,7 +1028,7 @@ bool ZLBFAM::AllocateBuffer(PGLOBAL g)
 
     // Some old tables can have PlugDB in their header
     if (strcmp(To_Buf, "PlugDB")) {
-      snprintf(g->Message, sizeof(g->Message), MSG(BAD_HEADER), Tdbp->GetFile(g));
+      sprintf(g->Message, MSG(BAD_HEADER), Tdbp->GetFile(g));
       return TRUE;
       } // endif strcmp
 
@@ -1063,7 +1063,7 @@ int ZLBFAM::GetNextPos(void)
 /***********************************************************************/
 bool ZLBFAM::SetPos(PGLOBAL g, int pos __attribute__((unused)))
   {
-  snprintf(g->Message, sizeof(g->Message), MSG(NO_SETPOS_YET), "GZ");
+  sprintf(g->Message, MSG(NO_SETPOS_YET), "GZ");
   return true;
 #if 0 // All this must be checked
   if (pos < 0) {
@@ -1146,7 +1146,7 @@ int ZLBFAM::ReadBuffer(PGLOBAL g)
     // fseek is required only in non sequential reading
     if (CurBlk != OldBlk + 1)
       if (fseek(Stream, Fpos, SEEK_SET)) {
-        snprintf(g->Message, sizeof(g->Message), MSG(FSETPOS_ERROR), Fpos);
+        sprintf(g->Message, MSG(FSETPOS_ERROR), Fpos);
         return RC_FX;
         } // endif fseek
 
@@ -1208,9 +1208,9 @@ int ZLBFAM::ReadBuffer(PGLOBAL g)
 
  err:
 #if defined(UNIX)
-  snprintf(g->Message, sizeof(g->Message), MSG(READ_ERROR), To_File, strerror(errno));
+  sprintf(g->Message, MSG(READ_ERROR), To_File, strerror(errno));
 #else
-  snprintf(g->Message, sizeof(g->Message), MSG(READ_ERROR), To_File, _strerror(NULL));
+  sprintf(g->Message, MSG(READ_ERROR), To_File, _strerror(NULL));
 #endif
   return RC_FX;
   } // end of ReadBuffer
@@ -1226,7 +1226,7 @@ int ZLBFAM::ReadCompressedBuffer(PGLOBAL g, void *rdbuf)
     num_read++;
 
     if (Optimized && BlkLen != signed(*Zlenp + sizeof(int))) {
-      snprintf(g->Message, sizeof(g->Message), MSG(BAD_BLK_SIZE), CurBlk + 1);
+      sprintf(g->Message, MSG(BAD_BLK_SIZE), CurBlk + 1);
       return RC_NF;
       } // endif BlkLen
 
@@ -1239,9 +1239,9 @@ int ZLBFAM::ReadCompressedBuffer(PGLOBAL g, void *rdbuf)
 
     if (zrc != Z_OK) {
       if (Zstream->msg)
-        snprintf(g->Message, sizeof(g->Message), MSG(FUNC_ERR_S), "inflate", Zstream->msg);
+        sprintf(g->Message, MSG(FUNC_ERR_S), "inflate", Zstream->msg);
       else
-        snprintf(g->Message, sizeof(g->Message), MSG(FUNCTION_ERROR), "inflate", (int)zrc);
+        sprintf(g->Message, MSG(FUNCTION_ERROR), "inflate", (int)zrc);
 
       return RC_NF;
       } // endif zrc
@@ -1329,9 +1329,9 @@ bool ZLBFAM::WriteCompressedBuffer(PGLOBAL g)
 
   if (zrc != Z_OK) {
     if (Zstream->msg)
-      snprintf(g->Message, sizeof(g->Message), MSG(FUNC_ERR_S), "deflate", Zstream->msg);
+      sprintf(g->Message, MSG(FUNC_ERR_S), "deflate", Zstream->msg);
     else
-      snprintf(g->Message, sizeof(g->Message), MSG(FUNCTION_ERROR), "deflate", (int)zrc);
+      sprintf(g->Message, MSG(FUNCTION_ERROR), "deflate", (int)zrc);
 
     return TRUE;
   } else
@@ -1341,7 +1341,7 @@ bool ZLBFAM::WriteCompressedBuffer(PGLOBAL g)
   BlkLen = *Zlenp + sizeof(int);
 
   if (fwrite(Zlenp, 1, BlkLen, Stream) != (size_t)BlkLen) {
-    snprintf(g->Message, sizeof(g->Message), MSG(FWRITE_ERROR), strerror(errno));
+    sprintf(g->Message, MSG(FWRITE_ERROR), strerror(errno));
     return TRUE;
     } // endif size
 
@@ -1356,7 +1356,7 @@ void ZLBFAM::CloseTableFile(PGLOBAL g, bool)
   int rc = RC_OK;
 
   if (Tdbp->GetMode() == MODE_INSERT) {
-    LPCSTR  name __attribute__((unused))= Tdbp->GetName();
+    (void) Tdbp->GetName();                     // XXX Should be removed?
     PDOSDEF defp = (PDOSDEF)Tdbp->GetDef();
 
     // Closing is True if last Write was in error
